﻿/* ========================================================================== */
/*!
 * 
 */
/* ========================================================================== */
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Collections;
using magicAnime;

namespace RecTestSched
{
    public class RecTestSched : Scheduler
    {
        //!
        static private RecTestSched mAcigPlugin;

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        static public RecTestSched GetInstance() { return mAcigPlugin; }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public RecTestSched()
        {
            General.myExceptionExit = false;
            try
            {
                mAcigPlugin = this;
                ScheduleForm.GetInstance().Initialize();
                SchedulerMain.GetInstance().Initialize();
            }
            catch (Exception exp)
            {
                General.myExceptionExit = true;
                General.LogOut(0, "プラグインが強制終了しました！ " + exp.Message);
                General.OutputString(exp.StackTrace, exp.Message);
                Release();
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        ~RecTestSched()
        {
            Release();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public override void Close()
        {
            try
            {
                SchedulerMain.OnExit();
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
            
            try
            {
                General.OnExit();
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
            
            try
            {
                ScheduleForm.OnExit();
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }

            base.Close();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void Release()
        {
            try
            {
                SchedulerMain.DelInstance();
                General.DelRecorderTable();
                ScheduleForm.DelInstance();
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////
        // プラグインプロパティ
        ////////////////////////////////////////////////////////////////////////////////
        public override AbilityFlag Ability
        {
            get
            {
                return  AbilityFlag.MakeReservation |
                        AbilityFlag.ChangeReservation |
                        AbilityFlag.CancelReservation |
                        AbilityFlag.ExistReservation;
            }
        }
        public override string Extension { get { return ".ts"; } }
        public override string Folder { get { return base.Folder; } }
        public override string Name { get { return "RecTest Scheduler"; } }
        public override bool SubDirectory { get { return false; } }
        public override Type ProfilePageType { get { return null; } }
        public override Type ProfileType { get { return null; } }

        ////////////////////////////////////////////////////////////////////////////////
        // FUNCTION    :    mAgicScheduler::Flush
        // ABSTRACT    :    予約データのフラッシュを行う
        ////////////////////////////////////////////////////////////////////////////////
        public override void Flush()
        {
        }

        ////////////////////////////////////////////////////////////////////////////////
        // FUNCTION    :    mAgicScheduler::MakeReservation
        // ABSTRACT    :    時間を指定して録画予約を新規登録する
        // NOTE        :    同時録画できる環境では、時間重複しても失敗するとは限らない。
        //                  ただし、同じ録画タイトルの場合は失敗する。
        ////////////////////////////////////////////////////////////////////////////////
        public override void MakeReservation(
            string title,
            string uniqueID,
            string tvStation,
            DateTime dateTime,
            int minute,
            string Descript,
            uint groupCode,
            Profile param)
        {
            ScheduleForm.GetInstance().OnUpdateSchedule(title,
                                                        uniqueID,
                                                        tvStation,
                                                        dateTime,
                                                        minute,
                                                        Descript,
                                                        "",
                                                        true
                                                        );
        }

        ////////////////////////////////////////////////////////////////////////////////
        // FUNCTION    :    mAgicScheduler::ChangeReservation
        // ABSTRACT    :    指定された予約を変更する
        ////////////////////////////////////////////////////////////////////////////////
        public override void ChangeReservation(
            string title,
            string uniqueID,
            DateTime newDateTime,
            uint groupCode,
            Profile param)
        {
            ScheduleForm.GetInstance().OnUpdateSchedule(title,
                                                        uniqueID,
                                                        "",
                                                        newDateTime,
                                                        -1,
                                                        "",
                                                        "",
                                                        true
                                                        );
        }

        ////////////////////////////////////////////////////////////////////////////////
        // FUNCTION    :    mAgicScheduler::CancelReservation
        // ABSTRACT    :    指定された予約をキャンセルする
        ////////////////////////////////////////////////////////////////////////////////
        public override void CancelReservation(
            string title,
            string uniqueID)
        {
            ScheduleForm.GetInstance().OnUnloadSchedule(title, uniqueID);
        }

        ////////////////////////////////////////////////////////////////////////////////
        // FUNCTION    :    mAgicScheduler::ExistReservation
        // ABSTRACT    :    指定された予約が在るか確認する
        ////////////////////////////////////////////////////////////////////////////////
        public override bool ExistReservation(
            string title,
            string uniqueID)
        {
            return ScheduleForm.GetInstance().OnSearchSchedule(title, uniqueID);
        }

        ////////////////////////////////////////////////////////////////////////////////
        // FUNCTION    :    mAgicScheduler::GetStations
        // ABSTRACT    :    放送局一覧を返す
        ////////////////////////////////////////////////////////////////////////////////
        public override List<string> GetStations()
        {
            List<string> result = new List<string>();
            ScheduleForm.GetInstance().OnUpdateChannel(result);
            return result;
        }

        ////////////////////////////////////////////////////////////////////////////////
        // FUNCTION    :    mAgicScheduler::LogOut
        // ABSTRACT    :    ログを出力する
        ////////////////////////////////////////////////////////////////////////////////
        internal void LogOut(string _str)
        {
            Logger.Output("[RecTestSched] " + _str);
        }

    }
}
