﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;

namespace RecTestSched
{
    internal class SpinupDisc : IDisposable 
    {
        //!
        private string myPath;
        //!
        private System.Threading.Timer myTimer;
        //!
        private int myCount;
        //!
        private const int TIMER_WRITE_UPDATE = 5 * 100;

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public SpinupDisc( string _path )
        {
            string dirPath = Path.GetDirectoryName(_path);
            string fileName = Path.GetFileNameWithoutExtension(_path);
            myPath = dirPath + fileName + "_.spinup";
            myCount = 0;
            try
            {
                TimerCallback timerCallBack = new System.Threading.TimerCallback(TimerProc);
                myTimer = new System.Threading.Timer(timerCallBack, null, 0, TIMER_WRITE_UPDATE);
            }
            catch (Exception exp)
            {
                Release();
                General.LogOut(0, "内部プロセスの起動に失敗しました " + myPath);
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        ~SpinupDisc()
        {
            Release();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public void Dispose()
        {
            Release();
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void Release()
        {
            try
            {
                if (myTimer != null)
                {
                    General.TimerPause(myTimer);
                    myTimer.Dispose();
                    myTimer = null;
                }
                if (File.Exists(myPath))
                {
                    File.Delete(myPath);
                }
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void TimerChange()
        {
            try
            {
                if (myTimer != null)
                {
                    myTimer.Change(TIMER_WRITE_UPDATE, TIMER_WRITE_UPDATE);
                }
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void TimerProc(object param)
        {
            General.TimerPause(myTimer);

            try
            {
                string dirPath = Path.GetDirectoryName(myPath);
                if (Directory.Exists(dirPath) == false)
                {
                    Directory.CreateDirectory(dirPath);
                }

                string txtContents = "RecTest Scheduler for mAgicAnime.NET";
                for (int loop = 0; loop < 55; ++loop)
                {
                    txtContents += Environment.NewLine
                                 + "RecTest Scheduler for mAgicAnime.NET";  
                }
                txtContents += Environment.NewLine;

                byte[] byteArray = Encoding.Default.GetBytes(txtContents);

                using (FileStream fsWriter = new FileStream(myPath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                {
                    fsWriter.Seek(0,SeekOrigin.End);
                    fsWriter.Write(byteArray, 0, byteArray.Length);
                    fsWriter.Flush();
                    fsWriter.Close();
                }
            }
            catch (Exception exp)
            {
                General.LogOut(2, myPath + " の書き込みに失敗しました");
                General.OutputString(exp.StackTrace, exp.Message);
            }

            if (++myCount < 5)
            {
                TimerChange();
            }
            else
            {
                this.Dispose();
            }
        }

    }
}
