﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Collections;
using System.Xml;
using System.Reflection;

namespace RecTestSchediEPG
{
    internal class Config
    {
        //!
        [DllImport("User32.dll", EntryPoint = "FindWindow")]
        public static extern IntPtr FindWindow(String lpClassName, String lpWindowName);
        //!
        static private Config myThis;
        //!
        static internal string myAppPath;
        //!
        static internal string myOpenFile;
        //!
        static internal IntPtr myWindowHandle;

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        static public Config GetInstance()
        {
            if (myThis == null)
            {
                myThis = new Config();
            }
            return myThis;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        public Config()
        {
            try
            {
                myWindowHandle = IntPtr.Zero;

                //myAppPath = Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName);
                string execDir = Path.GetDirectoryName(Process.GetCurrentProcess().MainModule.FileName);
                myAppPath = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData)
                          + "\\magicAnime\\RecTestSched_"
                          + General.ToMd5(execDir)
                          + "\\";
                try
                {
                    if (Directory.Exists(myAppPath) == false)
                    {
                        Directory.CreateDirectory(myAppPath);
                        string refer = ";The following exe file refers to this directory. "
                                     + Environment.NewLine
                                     + execDir
                                     + Environment.NewLine
                                     ;
                        File.WriteAllText(Path.Combine(myAppPath, @"refer.txt"), refer);
                    }
                    General.Move(execDir,myAppPath,@"iEPGSchedConfig.xml");
                }
                catch (Exception exp)
                {
                    General.OutputString(exp.StackTrace, exp.Message);
                }

                myOpenFile = "";
                string[] strArgs = Environment.GetCommandLineArgs();
                foreach (string arg in strArgs)
                {
                    if (arg == Process.GetCurrentProcess().MainModule.FileName) { continue; }
                    if (myOpenFile != "") { myOpenFile += " "; }
                    myOpenFile += arg;
                }

            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        ~Config()
        {
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal bool FindScheduleWindow()
        {
            IntPtr hWnd;
            AssemblyName asmName = Assembly.GetExecutingAssembly().GetName();
            string windowText = "RecTest Scheduler for mAgicAnime.NET"
                      + "  Ver." + asmName.Version.Major + "." + asmName.Version.Minor + asmName.Version.Build;
            hWnd = FindWindow(null, windowText);
            if (hWnd != IntPtr.Zero)
            {
                myWindowHandle = hWnd;
                return true;
            }
            hWnd = FindWindow("WindowsForms10.Window.8.app.0.218f99c", null);
            if (hWnd != IntPtr.Zero)
            {
                myWindowHandle = hWnd;
                return true;
            }
            hWnd = FindWindow("WindowsForms10.Window.8.app.0.33c0d9d", null);
            if (hWnd != IntPtr.Zero)
            {
                myWindowHandle = hWnd;
                return true;
            }
            return false;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal bool Check()
        {
            if (myOpenFile == "")
            {
                return false;
            }
            if (myWindowHandle == IntPtr.Zero)
            {
                return false;
            }
            return true;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal string AppPath()
        {
            return myAppPath;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal string OpenPath()
        {
            return myOpenFile;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal bool Import()
        {
            bool beOK = false;
            try
            {
                using (XmlReader xmlReader = new XmlTextReader(Path.Combine(myAppPath, @"iEPGSchedConfig.xml")))
                {
                    beOK = true;
                    while (xmlReader.Read())
                    {
                        switch (xmlReader.NodeType)
                        {
                            case XmlNodeType.Element:
                                if (!InportConfigElement(xmlReader))
                                {
                                    beOK = false;
                                }
                                break;
                        }
                    }
                    xmlReader.Close();
                }
            }
            catch (Exception exp)
            {
                beOK = false;
                General.OutputString(exp.StackTrace, exp.Message);
            }
            return beOK;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private bool InportConfigElement(XmlReader _xml)
        {
            switch (_xml.Name)
            {
                case "Channel": return InportConfigChannel(_xml);
            }
            // 知らないタグは無視する
            return true;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private bool InportConfigChannel(XmlReader _xml)
        {
            string iEPG = "";
            string station = "";
            string start = "";
            string length = "";
            while (_xml.Read())
            {
                switch (_xml.NodeType)
                {
                    case XmlNodeType.Element:
                        switch (_xml.LocalName)
                        {
                            case "iEPG": iEPG = _xml.ReadString(); break;
                            case "Station": station = _xml.ReadString(); break;
                            case "Start": start = _xml.ReadString(); break;
                            case "Length": length = _xml.ReadString(); break;
                        }
                        break;

                    case XmlNodeType.EndElement:
                        switch (_xml.LocalName)
                        {
                            case "Entry":
                                if (iEPG != "" && station != "")
                                {
                                    Channel.Update( iEPG
                                                  , station
                                                  , General.ToInt32(start)
                                                  , General.ToInt32(length)
                                                  );
                                               
                                }
                                break;

                            case "Channel":
                                return true;
                        }
                        break;
                }
            }
            return false;
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        internal void Export()
        {
            try
            {
                using (XmlTextWriter xmlWriter = new XmlTextWriter(Path.Combine(myAppPath, @"iEPGSchedConfig.xml"), System.Text.Encoding.UTF8))
                {
                    xmlWriter.Formatting = System.Xml.Formatting.Indented;
                    xmlWriter.WriteStartDocument(true);
                    xmlWriter.WriteStartElement("Config");
                    {
                        ExportConfigWrite(xmlWriter);
                    }
                    xmlWriter.WriteEndElement();
                    xmlWriter.WriteEndDocument();
                    xmlWriter.Close();
                }
            }
            catch (Exception exp)
            {
                General.OutputString(exp.StackTrace, exp.Message);
            }
        }

        /* ========================================================================== */
        /*!
         */
        /* ========================================================================== */
        private void ExportConfigWrite(XmlTextWriter _xml)
        {
            _xml.WriteStartElement("Channel");
            {
                foreach (Channel ch in Channel.List())
                {
                    _xml.WriteStartElement("Entry");
                    {
                        _xml.WriteElementString("iEPG", ch.myiEPGStation);
                        _xml.WriteElementString("Station", ch.myRecStation);
                        _xml.WriteElementString("Start", ch.myAddStart.ToString());
                        _xml.WriteElementString("Length", ch.myAddLength.ToString());
                    }
                    _xml.WriteEndElement();
                }
            }
            _xml.WriteEndElement();
        }

    }
}
